@@ -54,13 +54,7 @@ module SDL.Audio
   , getAudioDeviceNames
 
     -- * 'AudioSpec'
-  , AudioSpec
-  , audioSpecFreq
-  , audioSpecFormat
-  , audioSpecChannels
-  , audioSpecSilence
-  , audioSpecSize
-  , audioSpecCallback
+  , AudioSpec(..)
 
     -- * Audio Drivers
   , getAudioDrivers
@@ -104,6 +98,12 @@ import Data.Foldable (Foldable)
 import Data.Traversable (Traversable)
 #endif
 
+#if MIN_VERSION_base(4,12,0)
+import Data.Kind (Type)
+#else
+# define Type *
+#endif
+
 {-
 
 $audioDevice
@@ -175,9 +175,9 @@ openAudioDevice OpenDeviceSpec{..} = liftIO $
         return (audioDevice, spec)
 
   where
-  changes = foldl (.|.) 0 [ foldChangeable (const Raw.SDL_AUDIO_ALLOW_FREQUENCY_CHANGE) (const 0) openDeviceFreq
-                          , foldChangeable (const Raw.SDL_AUDIO_ALLOW_FORMAT_CHANGE) (const 0) openDeviceFormat
-                          , foldChangeable (const Raw.SDL_AUDIO_ALLOW_CHANNELS_CHANGE) (const 0) openDeviceChannels
+  changes = foldl (.|.) 0 [ foldChangeable (const 0) (const Raw.SDL_AUDIO_ALLOW_FREQUENCY_CHANGE) openDeviceFreq
+                          , foldChangeable (const 0) (const Raw.SDL_AUDIO_ALLOW_FORMAT_CHANGE) openDeviceFormat
+                          , foldChangeable (const 0) (const Raw.SDL_AUDIO_ALLOW_CHANNELS_CHANGE) openDeviceChannels
                           ]
 
   channelsToWord8 Mono = 1
@@ -219,7 +219,7 @@ openAudioDevice OpenDeviceSpec{..} = liftIO $
   audioFormatStorable FloatingBEAudio = Dict
   audioFormatStorable FloatingNativeAudio = Dict
 
-data Dict :: Constraint -> * where
+data Dict :: Constraint -> Type where
   Dict :: c => Dict c
 
 -- |
