@@ -109,10 +109,10 @@ import qualified SDL.Raw as Raw
 import Control.Applicative
 #endif
 
--- | A single SDL event. This event occured at 'eventTimestamp' and carries data under 'eventPayload'.
+-- | A single SDL event. This event occurred at 'eventTimestamp' and carries data under 'eventPayload'.
 data Event = Event
   { eventTimestamp :: Timestamp
-    -- ^ The time the event occured.
+    -- ^ The time the event occurred.
   , eventPayload :: EventPayload
     -- ^ Data pertaining to this event.
   } deriving (Eq, Ord, Generic, Show, Typeable)
@@ -396,7 +396,7 @@ data JoyButtonEventData =
 data JoyDeviceEventData =
   JoyDeviceEventData {joyDeviceEventConnection :: !JoyDeviceConnection
                       -- ^ Was the device added or removed?
-                     ,joyDeviceEventWhich :: !Int32
+                     ,joyDeviceEventWhich :: !Raw.JoystickID
                       -- ^ The instance id of the joystick that reported the event.
                      }
   deriving (Eq,Ord,Generic,Show,Typeable)
@@ -405,7 +405,7 @@ data JoyDeviceEventData =
 data ControllerAxisEventData =
   ControllerAxisEventData {controllerAxisEventWhich :: !Raw.JoystickID
                            -- ^ The joystick instance ID that reported the event.
-                          ,controllerAxisEventAxis :: !Word8
+                          ,controllerAxisEventAxis :: !ControllerAxis
                            -- ^ The index of the axis.
                           ,controllerAxisEventValue :: !Int16
                            -- ^ The axis value ranging between -32768 and 32767.
@@ -427,7 +427,7 @@ data ControllerButtonEventData =
 data ControllerDeviceEventData =
   ControllerDeviceEventData {controllerDeviceEventConnection :: !ControllerDeviceConnection
                              -- ^ Was the device added, removed, or remapped?
-                            ,controllerDeviceEventWhich :: !Int32
+                            ,controllerDeviceEventWhich :: !Raw.JoystickID
                              -- ^ The joystick instance ID that reported the event.
                             }
   deriving (Eq,Ord,Generic,Show,Typeable)
@@ -681,11 +681,15 @@ convertRaw (Raw.JoyButtonEvent _ ts a b c) =
 convertRaw (Raw.JoyDeviceEvent t ts a) =
   return (Event ts (JoyDeviceEvent (JoyDeviceEventData (fromNumber t) a)))
 convertRaw (Raw.ControllerAxisEvent _ ts a b c) =
-  return (Event ts (ControllerAxisEvent (ControllerAxisEventData a b c)))
+  return (Event ts
+            (ControllerAxisEvent
+              (ControllerAxisEventData a
+                                      (fromNumber $ fromIntegral b)
+                                      c)))
 convertRaw (Raw.ControllerButtonEvent t ts a b _) =
-  return (Event ts 
+  return (Event ts
            (ControllerButtonEvent
-             (ControllerButtonEventData a 
+             (ControllerButtonEventData a
                                         (fromNumber $ fromIntegral b)
                                         (fromNumber t))))
 convertRaw (Raw.ControllerDeviceEvent t ts a) =
@@ -754,9 +758,9 @@ pollEvent =
     if n == 0
       then return Nothing
       else alloca $ \e -> do
-             n <- Raw.pollEvent e
+             n' <- Raw.pollEvent e
              -- Checking 0 again doesn't hurt and it's good to be safe.
-             if n == 0
+             if n' == 0
                then return Nothing
                else fmap Just (peek e >>= convertRaw)
 
@@ -764,12 +768,21 @@ pollEvent =
 --
 -- Like 'pollEvent' this function should only be called in the OS thread which
 -- set the video mode.
-pollEvents :: (Functor m, MonadIO m) => m [Event]
-pollEvents =
-  do e <- pollEvent
-     case e of
-       Nothing -> return []
-       Just e' -> (e' :) <$> pollEvents
+pollEvents :: MonadIO m => m [Event]
+pollEvents = liftIO $ do
+  Raw.pumpEvents
+  peepAllEvents >>= mapM convertRaw where
+    peepAllEvents = do
+      numPeeped <- Raw.peepEvents
+        Raw.eventBuffer
+        Raw.eventBufferSize
+        Raw.SDL_GETEVENT
+        Raw.SDL_FIRSTEVENT
+        Raw.SDL_LASTEVENT
+      peeped <- peekArray (fromIntegral numPeeped) Raw.eventBuffer
+      if numPeeped == Raw.eventBufferSize -- are there more events to peep?
+        then (peeped ++) <$> peepAllEvents
+        else return peeped
 
 -- | Run a monadic computation, accumulating over all known 'Event's.
 --
@@ -862,7 +875,7 @@ registerEvent registeredEventDataToEvent eventToRegisteredEventData = do
               0 -> return $ EventPushFiltered
               _ -> EventPushFailure <$> getError
 
-        getEv (Event ts (UserEvent (UserEventData typ mWin code d1 d2))) =
+        getEv (Event ts (UserEvent (UserEventData _typ mWin code d1 d2))) =
           registeredEventDataToEvent (RegisteredEventData mWin code d1 d2) ts
         getEv _ = return Nothing
 
@@ -911,6 +924,6 @@ delEventWatch = liftIO . runEventWatchRemoval
 
 -- | Checks raw Windows for null references.
 getWindowFromID :: MonadIO m => Word32 -> m (Maybe Window)
-getWindowFromID id = do
-  rawWindow <- Raw.getWindowFromID id
+getWindowFromID windowId = do
+  rawWindow <- Raw.getWindowFromID windowId
   return $ if rawWindow == nullPtr then Nothing else Just $ Window rawWindow
