@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+
 module SDL.Raw.Video (
   -- * Display and Window Management
   createWindow,
@@ -32,11 +34,13 @@ module SDL.Raw.Video (
   getDisplayDPI,
   getDisplayMode,
   getDisplayName,
+  getDisplayUsableBounds,
   getGrabbedWindow,
   getNumDisplayModes,
   getNumVideoDisplays,
   getNumVideoDrivers,
   getVideoDriver,
+  getWindowBordersSize,
   getWindowBrightness,
   getWindowData,
   getWindowDisplayIndex,
@@ -48,6 +52,7 @@ module SDL.Raw.Video (
   getWindowID,
   getWindowMaximumSize,
   getWindowMinimumSize,
+  getWindowOpacity,
   getWindowPixelFormat,
   getWindowPosition,
   getWindowSize,
@@ -69,6 +74,7 @@ module SDL.Raw.Video (
   setWindowIcon,
   setWindowMaximumSize,
   setWindowMinimumSize,
+  setWindowOpacity,
   setWindowPosition,
   setWindowSize,
   setWindowTitle,
@@ -81,6 +87,7 @@ module SDL.Raw.Video (
   videoQuit,
 
   -- * 2D Accelerated Rendering
+  composeCustomBlendMode,
   createRenderer,
   createSoftwareRenderer,
   createTexture,
@@ -112,6 +119,20 @@ module SDL.Raw.Video (
   renderFillRect,
   renderFillRectEx,
   renderFillRects,
+#ifdef RECENT_ISH
+  renderCopyF,
+  renderCopyExF,
+  renderDrawLineF,
+  renderDrawLinesF,
+  renderDrawPointF,
+  renderDrawPointsF,
+  renderDrawRectF,
+  renderDrawRectsF,
+  renderFillRectF,
+  renderFillRectsF,
+  renderGeometry,
+  renderGeometryRaw,
+#endif
   renderGetClipRect,
   renderGetLogicalSize,
   renderGetScale,
@@ -247,11 +268,13 @@ foreign import ccall "SDL.h SDL_GetDisplayBounds" getDisplayBoundsFFI :: CInt ->
 foreign import ccall "SDL.h SDL_GetDisplayDPI" getDisplayDPIFFI :: CInt -> Ptr CFloat -> Ptr CFloat -> Ptr CFloat -> IO CInt
 foreign import ccall "SDL.h SDL_GetDisplayMode" getDisplayModeFFI :: CInt -> CInt -> Ptr DisplayMode -> IO CInt
 foreign import ccall "SDL.h SDL_GetDisplayName" getDisplayNameFFI :: CInt -> IO CString
+foreign import ccall "SDL.h SDL_GetDisplayUsableBounds" getDisplayUsableBoundsFFI :: CInt -> Ptr Rect -> IO CInt
 foreign import ccall "SDL.h SDL_GetGrabbedWindow" getGrabbedWindowFFI :: IO Window
 foreign import ccall "SDL.h SDL_GetNumDisplayModes" getNumDisplayModesFFI :: CInt -> IO CInt
 foreign import ccall "SDL.h SDL_GetNumVideoDisplays" getNumVideoDisplaysFFI :: IO CInt
 foreign import ccall "SDL.h SDL_GetNumVideoDrivers" getNumVideoDriversFFI :: IO CInt
 foreign import ccall "SDL.h SDL_GetVideoDriver" getVideoDriverFFI :: CInt -> IO CString
+foreign import ccall "SDL.h SDL_GetWindowBordersSize" getWindowBordersSizeFFI :: Window -> Ptr CInt -> Ptr CInt -> Ptr CInt -> Ptr CInt -> IO CInt
 foreign import ccall "SDL.h SDL_GetWindowBrightness" getWindowBrightnessFFI :: Window -> IO CFloat
 foreign import ccall "SDL.h SDL_GetWindowData" getWindowDataFFI :: Window -> CString -> IO (Ptr ())
 foreign import ccall "SDL.h SDL_GetWindowDisplayIndex" getWindowDisplayIndexFFI :: Window -> IO CInt
@@ -263,6 +286,7 @@ foreign import ccall "SDL.h SDL_GetWindowGrab" getWindowGrabFFI :: Window -> IO
 foreign import ccall "SDL.h SDL_GetWindowID" getWindowIDFFI :: Window -> IO Word32
 foreign import ccall "SDL.h SDL_GetWindowMaximumSize" getWindowMaximumSizeFFI :: Window -> Ptr CInt -> Ptr CInt -> IO ()
 foreign import ccall "SDL.h SDL_GetWindowMinimumSize" getWindowMinimumSizeFFI :: Window -> Ptr CInt -> Ptr CInt -> IO ()
+foreign import ccall "SDL.h SDL_GetWindowOpacity" getWindowOpacityFFI :: Window -> Ptr CFloat -> IO ()
 foreign import ccall "SDL.h SDL_GetWindowPixelFormat" getWindowPixelFormatFFI :: Window -> IO Word32
 foreign import ccall "SDL.h SDL_GetWindowPosition" getWindowPositionFFI :: Window -> Ptr CInt -> Ptr CInt -> IO ()
 foreign import ccall "SDL.h SDL_GetWindowSize" getWindowSizeFFI :: Window -> Ptr CInt -> Ptr CInt -> IO ()
@@ -284,6 +308,7 @@ foreign import ccall "SDL.h SDL_SetWindowGrab" setWindowGrabFFI :: Window -> Boo
 foreign import ccall "SDL.h SDL_SetWindowIcon" setWindowIconFFI :: Window -> Ptr Surface -> IO ()
 foreign import ccall "SDL.h SDL_SetWindowMaximumSize" setWindowMaximumSizeFFI :: Window -> CInt -> CInt -> IO ()
 foreign import ccall "SDL.h SDL_SetWindowMinimumSize" setWindowMinimumSizeFFI :: Window -> CInt -> CInt -> IO ()
+foreign import ccall "SDL.h SDL_SetWindowOpacity" setWindowOpacityFFI :: Window -> CFloat -> IO ()
 foreign import ccall "SDL.h SDL_SetWindowPosition" setWindowPositionFFI :: Window -> CInt -> CInt -> IO ()
 foreign import ccall "SDL.h SDL_SetWindowSize" setWindowSizeFFI :: Window -> CInt -> CInt -> IO ()
 foreign import ccall "SDL.h SDL_SetWindowTitle" setWindowTitleFFI :: Window -> CString -> IO ()
@@ -295,6 +320,7 @@ foreign import ccall "SDL.h SDL_UpdateWindowSurfaceRects" updateWindowSurfaceRec
 foreign import ccall "SDL.h SDL_VideoInit" videoInitFFI :: CString -> IO CInt
 foreign import ccall "SDL.h SDL_VideoQuit" videoQuitFFI :: IO ()
 
+foreign import ccall "SDL.h SDL_ComposeCustomBlendMode" composeCustomBlendModeFFI :: BlendFactor -> BlendFactor -> BlendOperation -> BlendFactor -> BlendFactor -> BlendOperation -> IO BlendMode
 foreign import ccall "SDL.h SDL_CreateRenderer" createRendererFFI :: Window -> CInt -> Word32 -> IO Renderer
 foreign import ccall "SDL.h SDL_CreateSoftwareRenderer" createSoftwareRendererFFI :: Ptr Surface -> IO Renderer
 foreign import ccall "SDL.h SDL_CreateTexture" createTextureFFI :: Renderer -> Word32 -> CInt -> CInt -> CInt -> IO Texture
@@ -324,6 +350,20 @@ foreign import ccall "SDL.h SDL_RenderDrawPoints" renderDrawPointsFFI :: Rendere
 foreign import ccall "SDL.h SDL_RenderDrawRect" renderDrawRectFFI :: Renderer -> Ptr Rect -> IO CInt
 foreign import ccall "SDL.h SDL_RenderDrawRects" renderDrawRectsFFI :: Renderer -> Ptr Rect -> CInt -> IO CInt
 foreign import ccall "SDL.h SDL_RenderFillRect" renderFillRectFFI :: Renderer -> Ptr Rect -> IO CInt
+#ifdef RECENT_ISH
+foreign import ccall "SDL.h SDL_RenderCopyF" renderCopyFFFI :: Renderer -> Texture -> Ptr Rect -> Ptr FRect -> IO CInt
+foreign import ccall "SDL.h SDL_RenderCopyExF" renderCopyExFFFI :: Renderer -> Texture -> Ptr Rect -> Ptr FRect -> CDouble -> Ptr FPoint -> RendererFlip -> IO CInt
+foreign import ccall "SDL.h SDL_RenderDrawLineF" renderDrawLineFFFI :: Renderer -> CFloat -> CFloat -> CFloat -> CFloat -> IO CInt
+foreign import ccall "SDL.h SDL_RenderDrawLinesF" renderDrawLinesFFFI :: Renderer -> Ptr FPoint -> CInt -> IO CInt
+foreign import ccall "SDL.h SDL_RenderDrawPointF" renderDrawPointFFFI :: Renderer -> CFloat -> CFloat -> IO CInt
+foreign import ccall "SDL.h SDL_RenderDrawPointsF" renderDrawPointsFFFI :: Renderer -> Ptr FPoint -> CInt -> IO CInt
+foreign import ccall "SDL.h SDL_RenderDrawRectF" renderDrawRectFFFI :: Renderer -> Ptr FRect -> IO CInt
+foreign import ccall "SDL.h SDL_RenderDrawRectsF" renderDrawRectsFFFI :: Renderer -> Ptr FRect -> CInt -> IO CInt
+foreign import ccall "SDL.h SDL_RenderFillRectF" renderFillRectFFFI :: Renderer -> Ptr FRect -> IO CInt
+foreign import ccall "SDL.h SDL_RenderFillRectsF" renderFillRectsFFFI :: Renderer -> Ptr FRect -> CInt -> IO CInt
+foreign import ccall "SDL.h SDL_RenderGeometry" renderGeometryFFI :: Renderer -> Texture -> Ptr Vertex -> CInt -> Ptr CInt -> CInt -> IO CInt
+foreign import ccall "SDL.h SDL_RenderGeometryRaw" renderGeometryRawFFI :: Renderer -> Texture -> Ptr FPoint -> CInt -> Ptr Color -> CInt -> Ptr FPoint -> CInt -> CInt -> Ptr () -> CInt -> CInt -> IO CInt
+#endif
 foreign import ccall "sqlhelper.c SDLHelper_RenderFillRectEx" renderFillRectExFFI :: Renderer -> CInt -> CInt -> CInt -> CInt -> IO CInt
 foreign import ccall "SDL.h SDL_RenderFillRects" renderFillRectsFFI :: Renderer -> Ptr Rect -> CInt -> IO CInt
 foreign import ccall "SDL.h SDL_RenderGetClipRect" renderGetClipRectFFI :: Renderer -> Ptr Rect -> IO ()
@@ -539,6 +579,10 @@ getDisplayName :: MonadIO m => CInt -> m CString
 getDisplayName v1 = liftIO $ getDisplayNameFFI v1
 {-# INLINE getDisplayName #-}
 
+getDisplayUsableBounds :: MonadIO m => CInt -> Ptr Rect -> m CInt
+getDisplayUsableBounds v1 v2 = liftIO $ getDisplayUsableBoundsFFI v1 v2
+{-# INLINE getDisplayUsableBounds #-}
+
 getGrabbedWindow :: MonadIO m => m Window
 getGrabbedWindow = liftIO getGrabbedWindowFFI
 {-# INLINE getGrabbedWindow #-}
@@ -559,6 +603,10 @@ getVideoDriver :: MonadIO m => CInt -> m CString
 getVideoDriver v1 = liftIO $ getVideoDriverFFI v1
 {-# INLINE getVideoDriver #-}
 
+getWindowBordersSize :: MonadIO m => Window -> Ptr CInt -> Ptr CInt -> Ptr CInt -> Ptr CInt -> m CInt
+getWindowBordersSize v1 v2 v3 v4 v5 = liftIO $ getWindowBordersSizeFFI v1 v2 v3 v4 v5
+{-# INLINE getWindowBordersSize #-}
+
 getWindowBrightness :: MonadIO m => Window -> m CFloat
 getWindowBrightness v1 = liftIO $ getWindowBrightnessFFI v1
 {-# INLINE getWindowBrightness #-}
@@ -603,6 +651,10 @@ getWindowMinimumSize :: MonadIO m => Window -> Ptr CInt -> Ptr CInt -> m ()
 getWindowMinimumSize v1 v2 v3 = liftIO $ getWindowMinimumSizeFFI v1 v2 v3
 {-# INLINE getWindowMinimumSize #-}
 
+getWindowOpacity :: MonadIO m => Window -> Ptr CFloat -> m ()
+getWindowOpacity v1 v2 = liftIO $ getWindowOpacityFFI v1 v2
+{-# INLINE getWindowOpacity #-}
+
 getWindowPixelFormat :: MonadIO m => Window -> m Word32
 getWindowPixelFormat v1 = liftIO $ getWindowPixelFormatFFI v1
 {-# INLINE getWindowPixelFormat #-}
@@ -687,6 +739,10 @@ setWindowMinimumSize :: MonadIO m => Window -> CInt -> CInt -> m ()
 setWindowMinimumSize v1 v2 v3 = liftIO $ setWindowMinimumSizeFFI v1 v2 v3
 {-# INLINE setWindowMinimumSize #-}
 
+setWindowOpacity :: MonadIO m => Window -> CFloat -> m ()
+setWindowOpacity v1 v2 = liftIO $ setWindowOpacityFFI v1 v2
+{-# INLINE setWindowOpacity #-}
+
 setWindowPosition :: MonadIO m => Window -> CInt -> CInt -> m ()
 setWindowPosition v1 v2 v3 = liftIO $ setWindowPositionFFI v1 v2 v3
 {-# INLINE setWindowPosition #-}
@@ -727,6 +783,10 @@ videoQuit :: MonadIO m => m ()
 videoQuit = liftIO videoQuitFFI
 {-# INLINE videoQuit #-}
 
+composeCustomBlendMode :: MonadIO m => BlendFactor -> BlendFactor -> BlendOperation -> BlendFactor -> BlendFactor -> BlendOperation -> m BlendMode
+composeCustomBlendMode v1 v2 v3 v4 v5 v6 = liftIO $ composeCustomBlendModeFFI v1 v2 v3 v4 v5 v6
+{-# INLINE composeCustomBlendMode #-}
+
 createRenderer :: MonadIO m => Window -> CInt -> Word32 -> m Renderer
 createRenderer v1 v2 v3 = liftIO $ createRendererFFI v1 v2 v3
 {-# INLINE createRenderer #-}
@@ -851,6 +911,58 @@ renderFillRects :: MonadIO m => Renderer -> Ptr Rect -> CInt -> m CInt
 renderFillRects v1 v2 v3 = liftIO $ renderFillRectsFFI v1 v2 v3
 {-# INLINE renderFillRects #-}
 
+#ifdef RECENT_ISH
+
+renderCopyF :: MonadIO m => Renderer -> Texture -> Ptr Rect -> Ptr FRect -> m CInt
+renderCopyF v1 v2 v3 v4 = liftIO $ renderCopyFFFI v1 v2 v3 v4
+{-# INLINE renderCopyF #-}
+
+renderCopyExF :: MonadIO m => Renderer -> Texture -> Ptr Rect -> Ptr FRect -> CDouble -> Ptr FPoint -> RendererFlip -> m CInt
+renderCopyExF v1 v2 v3 v4 v5 v6 v7 = liftIO $ renderCopyExFFFI v1 v2 v3 v4 v5 v6 v7
+{-# INLINE renderCopyExF #-}
+
+renderDrawLineF :: MonadIO m => Renderer -> CFloat -> CFloat -> CFloat -> CFloat -> m CInt
+renderDrawLineF v1 v2 v3 v4 v5 = liftIO $ renderDrawLineFFFI v1 v2 v3 v4 v5
+{-# INLINE renderDrawLineF #-}
+
+renderDrawLinesF :: MonadIO m => Renderer -> Ptr FPoint -> CInt -> m CInt
+renderDrawLinesF v1 v2 v3 = liftIO $ renderDrawLinesFFFI v1 v2 v3
+{-# INLINE renderDrawLinesF #-}
+
+renderDrawPointF :: MonadIO m => Renderer -> CFloat -> CFloat -> m CInt
+renderDrawPointF v1 v2 v3 = liftIO $ renderDrawPointFFFI v1 v2 v3
+{-# INLINE renderDrawPointF #-}
+
+renderDrawPointsF :: MonadIO m => Renderer -> Ptr FPoint -> CInt -> m CInt
+renderDrawPointsF v1 v2 v3 = liftIO $ renderDrawPointsFFFI v1 v2 v3
+{-# INLINE renderDrawPointsF #-}
+
+renderDrawRectF :: MonadIO m => Renderer -> Ptr FRect -> m CInt
+renderDrawRectF v1 v2 = liftIO $ renderDrawRectFFFI v1 v2
+{-# INLINE renderDrawRectF #-}
+
+renderDrawRectsF :: MonadIO m => Renderer -> Ptr FRect -> CInt -> m CInt
+renderDrawRectsF v1 v2 v3 = liftIO $ renderDrawRectsFFFI v1 v2 v3
+{-# INLINE renderDrawRectsF #-}
+
+renderFillRectF :: MonadIO m => Renderer -> Ptr FRect -> m CInt
+renderFillRectF v1 v2 = liftIO $ renderFillRectFFFI v1 v2
+{-# INLINE renderFillRectF #-}
+
+renderFillRectsF :: MonadIO m => Renderer -> Ptr FRect -> CInt -> m CInt
+renderFillRectsF v1 v2 v3 = liftIO $ renderFillRectsFFFI v1 v2 v3
+{-# INLINE renderFillRectsF #-}
+
+renderGeometry :: MonadIO m => Renderer -> Texture -> Ptr Vertex -> CInt -> Ptr CInt -> CInt -> m CInt
+renderGeometry v1 v2 v3 v4 v5 v6 = liftIO $ renderGeometryFFI v1 v2 v3 v4 v5 v6
+{-# INLINE renderGeometry #-}
+
+renderGeometryRaw :: MonadIO m => Renderer -> Texture -> Ptr FPoint -> CInt -> Ptr Color -> CInt -> Ptr FPoint -> CInt -> CInt -> Ptr () -> CInt -> CInt -> m CInt
+renderGeometryRaw v1 v2 v3 v4 v5 v6 v7 v8 v9 v10 v11 v12 = liftIO $ renderGeometryRawFFI v1 v2 v3 v4 v5 v6 v7 v8 v9 v10 v11 v12
+{-# INLINE renderGeometryRaw #-}
+#endif
+
+
 renderGetClipRect :: MonadIO m => Renderer -> Ptr Rect -> m ()
 renderGetClipRect v1 v2 = liftIO $ renderGetClipRectFFI v1 v2
 {-# INLINE renderGetClipRect #-}
