@@ -25,6 +25,7 @@ module SDL.Video
   -- * Window Attributes
   , windowMinimumSize
   , windowMaximumSize
+  , windowOpacity
   , windowSize
   , windowBordered
   , windowBrightness
@@ -32,6 +33,8 @@ module SDL.Video
   , windowGrab
   , setWindowMode
   , getWindowAbsolutePosition
+  , getWindowBordersSize
+  , setWindowIcon
   , setWindowPosition
   , windowTitle
   , windowData
@@ -80,7 +83,7 @@ import Control.Monad.IO.Class (MonadIO, liftIO)
 import Data.Bits
 import Data.Data (Data)
 import Data.Foldable
-import Data.Maybe (isJust, fromMaybe)
+import Data.Maybe (fromMaybe)
 import Data.Monoid (First(..))
 import Data.Text (Text)
 import Data.Typeable
@@ -295,6 +298,11 @@ setWindowMode (Window w) mode =
       Minimized -> Raw.minimizeWindow w >> return 0
       Windowed -> Raw.setWindowFullscreen w 0 <* Raw.restoreWindow w
 
+-- | Set the icon for a window.
+setWindowIcon :: MonadIO m => Window -> Surface -> m ()
+setWindowIcon (Window win) (Surface sfc _) =
+  Raw.setWindowIcon win sfc
+
 -- | Set the position of the window.
 setWindowPosition :: MonadIO m => Window -> WindowPosition -> m ()
 setWindowPosition (Window w) pos = case pos of
@@ -311,6 +319,21 @@ getWindowAbsolutePosition (Window w) =
         Raw.getWindowPosition w wPtr hPtr
         V2 <$> peek wPtr <*> peek hPtr
 
+-- | Get the size of a window's borders (decorations) around the client area (top, left, bottom, right).
+--
+-- See @<https://wiki.libsdl.org/SDL_GetWindowBordersSize SDL_GetWindowBordersSize>@ for C documentation.
+getWindowBordersSize :: MonadIO m => Window -> m (Maybe (V4 CInt))
+getWindowBordersSize (Window win) =
+  liftIO $
+  alloca $ \tPtr ->
+  alloca $ \lPtr ->
+  alloca $ \bPtr ->
+  alloca $ \rPtr -> do
+    n <- Raw.getWindowBordersSize win tPtr lPtr bPtr rPtr
+    if n /= 0
+      then return Nothing
+      else fmap Just $ V4 <$> peek tPtr <*> peek lPtr <*> peek bPtr <*> peek rPtr
+
 -- | Get or set the size of a window's client area. Values beyond the maximum supported size are clamped.
 --
 -- If window was created with 'windowHighDPI' flag, this size may differ from the size in pixels.
@@ -589,6 +612,22 @@ windowMinimumSize (Window win) = makeStateVar getWindowMinimumSize setWindowMini
       Raw.getWindowMinimumSize win wptr hptr
       V2 <$> peek wptr <*> peek hptr
 
+-- | Get or set the opacity of a window.
+--
+-- This 'StateVar' can be modified using '$=' and the current value retrieved with 'get'.
+--
+-- See @<https://wiki.libsdl.org/SDL_SetWindowOpacity SDL_SetWindowOpacity>@ and @<https://wiki.libsdl.org/SDL_GetWindowOpacity SDL_GetWindowOpacity>@ for C documentation.
+windowOpacity :: Window -> StateVar CFloat
+windowOpacity (Window win) = makeStateVar getWindowOpacity setWindowOpacity
+  where
+  setWindowOpacity opacity = Raw.setWindowOpacity win opacity
+
+  getWindowOpacity =
+    liftIO $
+    alloca $ \optr -> do
+      Raw.getWindowOpacity win optr
+      peek optr
+
 createRenderer :: MonadIO m => Window -> CInt -> RendererConfig -> m Renderer
 createRenderer (Window w) driver config =
   liftIO . fmap Renderer $
