@@ -31,6 +31,21 @@ module SDL.Video.Renderer
   , drawRects
   , fillRect
   , fillRects
+#ifdef RECENT_ISH
+  , copyF
+  , copyExF
+  , drawLineF
+  , drawLinesF
+  , drawPointF
+  , drawPointsF
+  , drawRectF
+  , drawRectsF
+  , fillRectF
+  , fillRectsF
+  , renderGeometry
+  , Raw.Vertex(..)
+  , renderGeometryRaw
+#endif
   , present
 
   -- * 'Renderer' State
@@ -78,7 +93,7 @@ module SDL.Video.Renderer
   , paletteColors
   , paletteColor
   , PixelFormat(..)
-  , SurfacePixelFormat
+  , SurfacePixelFormat(..)
   , formatPalette
   , setPaletteColors
   , pixelFormatToMasks
@@ -119,7 +134,6 @@ module SDL.Video.Renderer
   ) where
 
 import Control.Monad.IO.Class (MonadIO, liftIO)
-import Control.Exception (catch, throw, SomeException, uninterruptibleMask_)
 import Data.Bits
 import Data.Data (Data)
 import Data.Foldable
@@ -222,14 +236,13 @@ updateTexture :: (Functor m, MonadIO m)
               -> Maybe (Rectangle CInt) -- ^ The area to update, Nothing for entire texture
               -> BS.ByteString -- ^ The raw pixel data
               -> CInt -- ^ The number of bytes in a row of pixel data, including padding between lines
-              -> m Texture
-updateTexture tex@(Texture t) rect pixels pitch = do
+              -> m ()
+updateTexture (Texture t) rect pixels pitch = do
   liftIO $ throwIfNeg_ "SDL.Video.updateTexture" "SDL_UpdateTexture" $
     maybeWith with rect $ \rectPtr ->
       let (pixelForeign, _, _) = BSI.toForeignPtr pixels
       in withForeignPtr pixelForeign $ \pixelsPtr ->
         Raw.updateTexture t (castPtr rectPtr) (castPtr pixelsPtr) pitch
-  return tex
 
 -- | Destroy the specified texture.
 --
@@ -579,7 +592,7 @@ data Rectangle a = Rectangle (Point V2 a) (V2 a)
 
 instance Storable a => Storable (Rectangle a) where
   sizeOf ~(Rectangle o s) = sizeOf o + sizeOf s
-  alignment _ = 0
+  alignment _ = alignment (undefined :: a)
   peek ptr = do
     o <- peek (castPtr ptr)
     s <- peek (castPtr (ptr `plusPtr` sizeOf o))
@@ -649,6 +662,133 @@ fillRects (Renderer r) rects = liftIO $
                           (castPtr rp)
                           (fromIntegral (SV.length rects))
 
+#ifdef RECENT_ISH
+
+-- | Copy a portion of the texture to the current rendering target.
+copyF :: MonadIO m => Renderer -> Texture -> Maybe (Rectangle CInt) -> Maybe (Rectangle CFloat) -> m ()
+copyF (Renderer r) (Texture t) srcRect dstRect = liftIO $
+  throwIfNeg_ "SDL.Video.copyF" "SDL_RenderCopyF" $
+    maybeWith with srcRect $ \src ->
+      maybeWith with dstRect $ \dst ->
+        Raw.renderCopyF r t (castPtr src) (castPtr dst)
+
+-- | Copy a portion of the texture to the current rendering target, optionally rotating it by angle around the given center and also flipping it top-bottom and/or left-right.
+copyExF :: MonadIO m
+       => Renderer -- ^ The rendering context
+       -> Texture -- ^ The source texture
+       -> Maybe (Rectangle CInt) -- ^ The source rectangle to copy, or 'Nothing' for the whole texture
+       -> Maybe (Rectangle CFloat) -- ^ The destination rectangle to copy to, or 'Nothing' for the whole rendering target. The texture will be stretched to fill the given rectangle.
+       -> CDouble -- ^ The angle of rotation in degrees. The rotation will be performed clockwise.
+       -> Maybe (Point V2 CFloat) -- ^ The point indicating the center of the rotation, or 'Nothing' to rotate around the center of the destination rectangle
+       -> V2 Bool -- ^ Whether to flip the texture on the X and/or Y axis
+       -> m ()
+copyExF (Renderer r) (Texture t) srcRect dstRect theta center flips =
+  liftIO $
+  throwIfNeg_ "SDL.Video.copyExF" "SDL_RenderCopyExF" $
+  maybeWith with srcRect $ \src ->
+  maybeWith with dstRect $ \dst ->
+  maybeWith with center $ \c ->
+  Raw.renderCopyExF r t (castPtr src) (castPtr dst) theta (castPtr c)
+                   (case flips of
+                      V2 x y -> (if x then Raw.SDL_FLIP_HORIZONTAL else 0) .|.
+                               (if y then Raw.SDL_FLIP_VERTICAL else 0))
+
+-- | Draw a line between two points on the current rendering target.
+drawLineF :: MonadIO m => Renderer -> Point V2 CFloat -> Point V2 CFloat -> m ()
+drawLineF (Renderer r) (P (V2 x y)) (P (V2 x' y')) = liftIO $
+  throwIfNeg_ "SDL.Video.drawLineF" "SDL_RenderDrawLineF" $
+    Raw.renderDrawLineF r x y x' y'
+
+-- | Draw a series of connected lines on the current rendering target.
+drawLinesF :: MonadIO m => Renderer -> SV.Vector (Point V2 CFloat) -> m ()
+drawLinesF (Renderer r) points = liftIO $
+  throwIfNeg_ "SDL.Video.drawLinesF" "SDL_RenderDrawLinesF" $
+    SV.unsafeWith points $ \p ->
+      Raw.renderDrawLinesF r (castPtr p) (fromIntegral (SV.length points))
+
+-- | Draw a point on the current rendering target.
+drawPointF :: MonadIO m => Renderer -> Point V2 CFloat -> m ()
+drawPointF (Renderer r) (P (V2 x y)) = liftIO $
+  throwIfNeg_ "SDL.Video.drawPointF" "SDL_RenderDrawPointF" $
+    Raw.renderDrawPointF r x y
+
+-- | Draw a collection of points on the current rendering target.
+drawPointsF :: MonadIO m => Renderer -> SV.Vector (Point V2 CFloat) -> m ()
+drawPointsF (Renderer r) points = liftIO $
+  throwIfNeg_ "SDL.Video.drawPointsF" "SDL_RenderDrawPointsF" $
+    SV.unsafeWith points $ \p ->
+      Raw.renderDrawPointsF r (castPtr p) (fromIntegral (SV.length points))
+
+-- | Draw the outline of a rectangle on the current rendering target.
+drawRectF :: MonadIO m => Renderer -> Rectangle CFloat -> m ()
+drawRectF (Renderer r) rect = liftIO $
+  throwIfNeg_ "SDL.Video.drawRectF" "SDL_RenderDrawRectF" $
+    with rect $ \rectPtr ->
+      Raw.renderDrawRectF r (castPtr rectPtr)
+
+-- | Draw a series of rectangle outlines on the current rendering target.
+drawRectsF :: MonadIO m => Renderer -> SV.Vector (Rectangle CFloat) -> m ()
+drawRectsF (Renderer r) rects = liftIO $
+  throwIfNeg_ "SDL.Video.drawRectsF" "SDL_RenderDrawRectsF" $
+    SV.unsafeWith rects $ \rp ->
+      Raw.renderDrawRectsF r (castPtr rp) (fromIntegral (SV.length rects))
+
+-- | Draw a filled rectangle on the current rendering target.
+fillRectF :: MonadIO m => Renderer -> Rectangle CFloat -> m ()
+fillRectF (Renderer r) rect = liftIO $
+  throwIfNeg_ "SDL.Video.fillRectF" "SDL_RenderFillRectF" $
+    with rect $ \rectPtr ->
+      Raw.renderFillRectF r (castPtr rectPtr)
+
+-- | Draw a series of filled rectangles on the current rendering target.
+fillRectsF :: MonadIO m => Renderer -> SV.Vector (Rectangle CFloat) -> m ()
+fillRectsF (Renderer r) rects = liftIO $
+  throwIfNeg_ "SDL.Video.fillRectsF" "SDL_RenderFillRectsF" $
+    SV.unsafeWith rects $ \rp ->
+      Raw.renderFillRectsF r (castPtr rp) (fromIntegral (SV.length rects))
+
+-- | Render a list of triangles, optionally using a texture and indices into the
+-- vertex array Color and alpha modulation is done per vertex
+-- (SDL_SetTextureColorMod and SDL_SetTextureAlphaMod are ignored).
+renderGeometry :: MonadIO m => Renderer -> Maybe Texture -> SV.Vector Raw.Vertex -> SV.Vector CInt -> m ()
+renderGeometry (Renderer r) mtexture vertices indices = liftIO $
+  throwIfNeg_ "SDL.Video.renderGeometry" "SDL_RenderGeometry" $
+    SV.unsafeWith vertices $ \vp ->
+      SV.unsafeWith indices $ \ip ->
+        Raw.renderGeometry r t vp (fromIntegral (SV.length vertices)) (ipOrNull ip) ipSize
+  where
+    t = case mtexture of
+      Just (Texture found) -> found
+      Nothing -> nullPtr
+
+    ipOrNull ip = if ipSize == 0 then nullPtr else ip
+
+    ipSize = fromIntegral (SV.length indices)
+
+-- | Render a list of triangles, optionally using a texture and indices into the
+-- vertex array Color and alpha modulation is done per vertex
+-- (SDL_SetTextureColorMod and SDL_SetTextureAlphaMod are ignored).
+--
+-- This version allows storeing vertex data in arbitrary types, but you have to provide
+-- pointers and strides yourself.
+renderGeometryRaw :: forall ix m . (Storable ix, MonadIO m) => Renderer -> Maybe Texture -> Ptr Raw.FPoint -> CInt -> Ptr Raw.Color -> CInt -> Ptr Raw.FPoint -> CInt -> CInt -> SV.Vector ix -> m ()
+renderGeometryRaw (Renderer r) mtexture xy xyStride color colorStride uv uvStride numVertices indices = liftIO $
+  throwIfNeg_ "SDL.Video.renderGeometryRaw" "SDL_RenderGeometryRaw" $
+    SV.unsafeWith indices $ \ip ->
+      Raw.renderGeometryRaw r t xy xyStride color colorStride uv uvStride numVertices (castPtr $ ipOrNull ip) ipSize sizeOfip
+  where
+    t = case mtexture of
+      Just (Texture found) -> found
+      Nothing -> nullPtr
+
+    ipOrNull ip = if ipSize == 0 then nullPtr else ip
+
+    ipSize = fromIntegral (SV.length indices)
+
+    sizeOfip = fromIntegral $ sizeOf (undefined :: ix)
+#endif
+
+
 -- | Clear the current rendering target with the drawing color.
 --
 -- See @<https://wiki.libsdl.org/SDL_RenderClear SDL_RenderClear>@ for C documentation.
@@ -903,7 +1043,7 @@ textureColorMod (Texture t) = makeStateVar getTextureColorMod setTextureColorMod
     Raw.setTextureColorMod t r g b
 
 data PixelFormat
-  = Unknown
+  = Unknown !Word32
   | Index1LSB
   | Index1MSB
   | Index4LSB
@@ -939,11 +1079,10 @@ data PixelFormat
   | YUY2
   | UYVY
   | YVYU
-  deriving (Bounded, Data, Enum, Eq, Generic, Ord, Read, Show, Typeable)
+  deriving (Data, Eq, Generic, Ord, Read, Show, Typeable)
 
 instance FromNumber PixelFormat Word32 where
   fromNumber n' = case n' of
-    Raw.SDL_PIXELFORMAT_UNKNOWN -> Unknown
     Raw.SDL_PIXELFORMAT_INDEX1LSB -> Index1LSB
     Raw.SDL_PIXELFORMAT_INDEX1MSB -> Index1MSB
     Raw.SDL_PIXELFORMAT_INDEX4LSB -> Index4LSB
@@ -979,11 +1118,12 @@ instance FromNumber PixelFormat Word32 where
     Raw.SDL_PIXELFORMAT_YUY2 -> YUY2
     Raw.SDL_PIXELFORMAT_UYVY -> UYVY
     Raw.SDL_PIXELFORMAT_YVYU -> YVYU
-    _ -> error "fromNumber: not numbered"
+    Raw.SDL_PIXELFORMAT_UNKNOWN -> Unknown n'
+    _ -> Unknown n'
 
 instance ToNumber PixelFormat Word32 where
   toNumber pf = case pf of
-    Unknown -> Raw.SDL_PIXELFORMAT_UNKNOWN
+    Unknown _ -> Raw.SDL_PIXELFORMAT_UNKNOWN
     Index1LSB -> Raw.SDL_PIXELFORMAT_INDEX1LSB
     Index1MSB -> Raw.SDL_PIXELFORMAT_INDEX1MSB
     Index4LSB -> Raw.SDL_PIXELFORMAT_INDEX4LSB
